const path = require("path")
const util = require("../util")
const webpack = require("webpack")
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const SpeedMeasurePlugin = require("speed-measure-webpack-plugin");
const merge = require('webpack-merge')

const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin')

const OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const devModuleRules = require("./webpack.config.dev").module.rules
const { userConfig, templates, webpackConfig } = require("./webpack.config.base")
const context = webpackConfig.context
let publicPath = userConfig.publicPath
const smp = new SpeedMeasurePlugin();
// const HappyPack = require('happypack');
// const happyThreadPool = HappyPack.ThreadPool({ size: os.cpus().length });

// 默认自己公司的配置
if (!publicPath) {
  let publicDomain = '//static.sunyur.com/'
  publicPath = publicDomain + path.basename(context).replace('style-', '') + '/'
}
// 异步模块打包 @todo 临时解决方案
const basePath = templates[0].basePath

let config = merge(webpackConfig, {
  output: Object.assign({}, webpackConfig.output, {
    publicPath: publicPath,
    // 输出的文件名 hash统一生成,chunkhash变化生成
    filename: '[name].js',
    // chunkFilename: basePath + '/[id].[chunkhash:8].js'
    chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].js' :basePath + '/[name].[chunkhash:8].js'
  }),
  module: {
    rules: userConfig.extractCSS ? webpackConfig.module.rules.concat(
      
      // vue
      {
        test: /\.vue$/,
        loader: 'vue-loader',
        options: {
          loaders: util.extractVueStlye(),
          preserveWhitespace: false
        }
      },
      // style
      util.stlyeRules(userConfig["devtool"], true)
    ) : devModuleRules
  },
  externals: {
    vue: 'Vue',
    'element-ui': 'Element',
    'vue-router': 'VueRouter',

  },
  performance: {
    maxEntrypointSize: 1000000,
    maxAssetSize: 1000000
  },
  mode: 'production',
  plugins: [
    // 配置环境
    new webpack.DefinePlugin({
      'process.env': {
        NODE_ENV: '"production"'
      },
      IS_DEV: JSON.stringify(false)
    }),
    new VueLoaderPlugin(),
    // 对css压缩
    new webpack.LoaderOptionsPlugin({
      minimize: true
    }),

    // 当模块内容没有修改时，保持module.id不变
    new webpack.HashedModuleIdsPlugin()

  ],
  // 对js进行压缩混淆 webpack 4
  optimization: {
    nodeEnv: 'production',
    splitChunks: {
      automaticNameDelimiter: '_'
    },
    minimize: true,
    concatenateModules: true,
    minimizer: [
        new UglifyJsPlugin({
            uglifyOptions: {
              compress: {
                warnings: false, // 在UglifyJs删除没有用到的代码时不输出警告
                drop_console: true, // 删除所有的 `console` 语句，可以兼容ie浏览器
                collapse_vars: true, // 内嵌定义了但是只用到一次的变量
                reduce_vars: true // 提取出出现多次但是没有定义成变量去引用的静态值
              }
            },
            sourceMap: true
        })
    ]
  },
  // 上线设置source-map会影响加载速度,但便于线上bug查找
  devtool: userConfig["devtool"],

  //编译期间一旦出错就停止编译
  bail: true
})

// webpack3.0  模块作用域提升
if (webpack.optimize.ModuleConcatenationPlugin) {
  config.plugins.push(new webpack.optimize.ModuleConcatenationPlugin())
}

// 提取文件的公共部分
if (userConfig && userConfig.vendor) {
  var filename = userConfig.chunkhash ? 'common.[chunkhash:' + userConfig.chunkhash + '].js' : 'common.js'
  templates.forEach((tpl, index) => {
    const common = tpl.basePath + '/common'
    const templateName = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename: templateName,
      template: tpl.template,
      chunks: [common, tpl.chunk],
      minify: {
        keepClosingSlash: true,
        removeAttributeQuotes: false,
        quoteCharacter: '"',
      },
      xhtml: false,
      inject: true
    }))
  })
  
} else{
  templates.forEach((tpl) => {
    const filename = path.join(tpl.basePath, 'index.html')
    config.plugins.push(new HtmlWebpackPlugin({
      filename,
      template: tpl.template,
      chunks: [tpl.chunk],
      inject: true
    }))
  })
}

if (userConfig.extractCSS) {
  config.plugins.push(
    // 提取css,生成对应对文件
    new MiniCssExtractPlugin({
      filename: userConfig.chunkhash ? '[name].[chunkhash:' + userConfig.chunkhash + '].css' : '[name].css',
      chunkFilename: userConfig.chunkhash ? basePath + '/[name].[chunkhash:' + userConfig.chunkhash + '].css' :basePath + '/[name].[chunkhash:8].css'
      // filename: '/[name].css',
    }),
    // 提取的css可能存在重复的样式，去掉重复的
    new OptimizeCSSPlugin({
      cssProcessorOptions: {
        safe: true
      }
    })
  )
}

// if (userConfig.index) {
//   let filename = path.join(commonsChunkPath, 'index.html')
//   config.plugins.push(
//     //引入模版文件
//     new HtmlWebpackPlugin({
//       filename: filename,
//       template: userConfig.index,
//       inject: true,
//     })
//   )
// }

if (userConfig.bundleAnalyzerReport) {
  var BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
  config.plugins.push(new BundleAnalyzerPlugin())
}
module.exports = smp.wrap(config)